@@ -1,6 +1,4 @@
-#!/usr/bin/python
-
-# Copyright (C) 2003-2004 Robey Pointer <robey@lag.net>
+# Copyright (C) 2003-2006 Robey Pointer <robey@lag.net>
 #
 # This file is part of paramiko.
 #
@@ -25,12 +23,13 @@ L{DSSKey}
 from Crypto.PublicKey import DSA
 from Crypto.Hash import SHA
 
-from common import *
-import util
-from ssh_exception import SSHException
-from message import Message
-from ber import BER, BERException
-from pkey import PKey
+from paramiko.common import *
+from paramiko import util
+from paramiko.ssh_exception import SSHException
+from paramiko.message import Message
+from paramiko.ber import BER, BERException
+from paramiko.pkey import PKey
+
 
 class DSSKey (PKey):
     """
@@ -38,7 +37,15 @@ class DSSKey (PKey):
     data.
     """
 
-    def __init__(self, msg=None, data=None, filename=None, password=None, vals=None):
+    def __init__(self, msg=None, data=None, filename=None, password=None, vals=None, file_obj=None):
+        self.p = None
+        self.q = None
+        self.g = None
+        self.y = None
+        self.x = None
+        if file_obj is not None:
+            self._from_private_key(file_obj, password)
+            return
         if filename is not None:
             self._from_private_key_file(filename, password)
             return
@@ -47,7 +54,9 @@ class DSSKey (PKey):
         if vals is not None:
             self.p, self.q, self.g, self.y = vals
         else:
-            if (msg is None) or (msg.get_string() != 'ssh-dss'):
+            if msg is None:
+                raise SSHException('Key object may not be empty')
+            if msg.get_string() != 'ssh-dss':
                 raise SSHException('Invalid key')
             self.p = msg.get_mpint()
             self.q = msg.get_mpint()
@@ -80,21 +89,28 @@ class DSSKey (PKey):
         return self.size
         
     def can_sign(self):
-        return hasattr(self, 'x')
+        return self.x is not None
 
-    def sign_ssh_data(self, randpool, data):
-        hash = SHA.new(data).digest()
+    def sign_ssh_data(self, rpool, data):
+        digest = SHA.new(data).digest()
         dss = DSA.construct((long(self.y), long(self.g), long(self.p), long(self.q), long(self.x)))
         # generate a suitable k
         qsize = len(util.deflate_long(self.q, 0))
-        while 1:
-            k = util.inflate_long(randpool.get_bytes(qsize), 1)
+        while True:
+            k = util.inflate_long(rpool.get_bytes(qsize), 1)
             if (k > 2) and (k < self.q):
                 break
-        r, s = dss.sign(util.inflate_long(hash, 1), k)
+        r, s = dss.sign(util.inflate_long(digest, 1), k)
         m = Message()
         m.add_string('ssh-dss')
-        m.add_string(util.deflate_long(r, 0) + util.deflate_long(s, 0))
+        # apparently, in rare cases, r or s may be shorter than 20 bytes!
+        rstr = util.deflate_long(r, 0)
+        sstr = util.deflate_long(s, 0)
+        if len(rstr) < 20:
+            rstr = '\x00' * (20 - len(rstr)) + rstr
+        if len(sstr) < 20:
+            sstr = '\x00' * (20 - len(sstr)) + sstr
+        m.add_string(rstr + sstr)
         return m
 
     def verify_ssh_sig(self, data, msg):
@@ -115,14 +131,22 @@ class DSSKey (PKey):
         dss = DSA.construct((long(self.y), long(self.g), long(self.p), long(self.q)))
         return dss.verify(sigM, (sigR, sigS))
 
-    def write_private_key_file(self, filename, password=None):
+    def _encode_key(self):
+        if self.x is None:
+            raise SSHException('Not enough key information')
         keylist = [ 0, self.p, self.q, self.g, self.y, self.x ]
         try:
             b = BER()
             b.encode(keylist)
         except BERException:
             raise SSHException('Unable to create ber encoding of key')
-        self._write_private_key_file('DSA', filename, str(b), password)
+        return str(b)
+
+    def write_private_key_file(self, filename, password=None):
+        self._write_private_key_file('DSA', filename, self._encode_key(), password)
+
+    def write_private_key(self, file_obj, password=None):
+        self._write_private_key('DSA', file_obj, self._encode_key(), password)
 
     def generate(bits=1024, progress_func=None):
         """
@@ -132,13 +156,12 @@ class DSSKey (PKey):
         @param bits: number of bits the generated key should be.
         @type bits: int
         @param progress_func: an optional function to call at key points in
-        key generation (used by C{pyCrypto.PublicKey}).
+            key generation (used by C{pyCrypto.PublicKey}).
         @type progress_func: function
         @return: new private key
         @rtype: L{DSSKey}
-
-        @since: fearow
         """
+        randpool.stir()
         dsa = DSA.generate(bits, randpool.get_bytes, progress_func)
         key = DSSKey(vals=(dsa.p, dsa.q, dsa.g, dsa.y))
         key.x = dsa.x
@@ -150,9 +173,16 @@ class DSSKey (PKey):
 
 
     def _from_private_key_file(self, filename, password):
+        data = self._read_private_key_file('DSA', filename, password)
+        self._decode_key(data)
+    
+    def _from_private_key(self, file_obj, password):
+        data = self._read_private_key('DSA', file_obj, password)
+        self._decode_key(data)
+    
+    def _decode_key(self, data):
         # private key file contains:
         # DSAPrivateKey = { version = 0, p, q, g, y, x }
-        data = self._read_private_key_file('DSA', filename, password)
         try:
             keylist = BER(data).decode()
         except BERException, x:
@@ -165,4 +195,3 @@ class DSSKey (PKey):
         self.y = keylist[4]
         self.x = keylist[5]
         self.size = util.bit_length(self.p)
-
