@@ -1,6 +1,4 @@
-#!/usr/bin/python
-
-# Copyright (C) 2003-2004 Robey Pointer <robey@lag.net>
+# Copyright (C) 2003-2006 Robey Pointer <robey@lag.net>
 #
 # This file is part of paramiko.
 #
@@ -26,12 +24,13 @@ from Crypto.PublicKey import RSA
 from Crypto.Hash import SHA, MD5
 from Crypto.Cipher import DES3
 
-from common import *
-from message import Message
-from ber import BER, BERException
-import util
-from pkey import PKey
-from ssh_exception import SSHException
+from paramiko.common import *
+from paramiko import util
+from paramiko.message import Message
+from paramiko.ber import BER, BERException
+from paramiko.pkey import PKey
+from paramiko.ssh_exception import SSHException
+
 
 class RSAKey (PKey):
     """
@@ -39,7 +38,15 @@ class RSAKey (PKey):
     data.
     """
 
-    def __init__(self, msg=None, data='', filename=None, password=None, vals=None):
+    def __init__(self, msg=None, data=None, filename=None, password=None, vals=None, file_obj=None):
+        self.n = None
+        self.e = None
+        self.d = None
+        self.p = None
+        self.q = None
+        if file_obj is not None:
+            self._from_private_key(file_obj, password)
+            return
         if filename is not None:
             self._from_private_key_file(filename, password)
             return
@@ -48,7 +55,9 @@ class RSAKey (PKey):
         if vals is not None:
             self.e, self.n = vals
         else:
-            if (msg is None) or (msg.get_string() != 'ssh-rsa'):
+            if msg is None:
+                raise SSHException('Key object may not be empty')
+            if msg.get_string() != 'ssh-rsa':
                 raise SSHException('Invalid key')
             self.e = msg.get_mpint()
             self.n = msg.get_mpint()
@@ -74,12 +83,12 @@ class RSAKey (PKey):
         return self.size
 
     def can_sign(self):
-        return hasattr(self, 'd')
+        return self.d is not None
 
-    def sign_ssh_data(self, randpool, data):
-        hash = SHA.new(data).digest()
+    def sign_ssh_data(self, rpool, data):
+        digest = SHA.new(data).digest()
         rsa = RSA.construct((long(self.n), long(self.e), long(self.d)))
-        sig = util.deflate_long(rsa.sign(self._pkcs1imify(hash), '')[0], 0)
+        sig = util.deflate_long(rsa.sign(self._pkcs1imify(digest), '')[0], 0)
         m = Message()
         m.add_string('ssh-rsa')
         m.add_string(sig)
@@ -88,15 +97,17 @@ class RSAKey (PKey):
     def verify_ssh_sig(self, data, msg):
         if msg.get_string() != 'ssh-rsa':
             return False
-        sig = util.inflate_long(msg.get_string(), 1)
+        sig = util.inflate_long(msg.get_string(), True)
         # verify the signature by SHA'ing the data and encrypting it using the
         # public key.  some wackiness ensues where we "pkcs1imify" the 20-byte
         # hash into a string as long as the RSA key.
-        hash = util.inflate_long(self._pkcs1imify(SHA.new(data).digest()), 1)
+        hash_obj = util.inflate_long(self._pkcs1imify(SHA.new(data).digest()), True)
         rsa = RSA.construct((long(self.n), long(self.e)))
-        return rsa.verify(hash, (sig,))
+        return rsa.verify(hash_obj, (sig,))
 
-    def write_private_key_file(self, filename, password=None):
+    def _encode_key(self):
+        if (self.p is None) or (self.q is None):
+            raise SSHException('Not enough key info to write private key file')
         keylist = [ 0, self.n, self.e, self.d, self.p, self.q,
                     self.d % (self.p - 1), self.d % (self.q - 1),
                     util.mod_inverse(self.q, self.p) ]
@@ -105,7 +116,13 @@ class RSAKey (PKey):
             b.encode(keylist)
         except BERException:
             raise SSHException('Unable to create ber encoding of key')
-        self._write_private_key_file('RSA', filename, str(b), password)
+        return str(b)
+
+    def write_private_key_file(self, filename, password=None):
+        self._write_private_key_file('RSA', filename, self._encode_key(), password)
+        
+    def write_private_key(self, file_obj, password=None):
+        self._write_private_key('RSA', file_obj, self._encode_key(), password)
 
     def generate(bits, progress_func=None):
         """
@@ -115,13 +132,12 @@ class RSAKey (PKey):
         @param bits: number of bits the generated key should be.
         @type bits: int
         @param progress_func: an optional function to call at key points in
-        key generation (used by C{pyCrypto.PublicKey}).
+            key generation (used by C{pyCrypto.PublicKey}).
         @type progress_func: function
         @return: new private key
         @rtype: L{RSAKey}
-
-        @since: fearow
         """
+        randpool.stir()
         rsa = RSA.generate(bits, randpool.get_bytes, progress_func)
         key = RSAKey(vals=(rsa.e, rsa.n))
         key.d = rsa.d
@@ -145,9 +161,16 @@ class RSAKey (PKey):
         return '\x00\x01' + filler + '\x00' + SHA1_DIGESTINFO + data
 
     def _from_private_key_file(self, filename, password):
+        data = self._read_private_key_file('RSA', filename, password)
+        self._decode_key(data)
+    
+    def _from_private_key(self, file_obj, password):
+        data = self._read_private_key('RSA', file_obj, password)
+        self._decode_key(data)
+    
+    def _decode_key(self, data):
         # private key file contains:
         # RSAPrivateKey = { version = 0, n, e, d, p, q, d mod p-1, d mod q-1, q**-1 mod p }
-        data = self._read_private_key_file('RSA', filename, password)
         try:
             keylist = BER(data).decode()
         except BERException:
@@ -161,4 +184,3 @@ class RSAKey (PKey):
         self.p = keylist[4]
         self.q = keylist[5]
         self.size = util.bit_length(self.n)
-
