@@ -1,4 +1,4 @@
-# Copyright (C) 2003-2005 Robey Pointer <robey@lag.net>
+# Copyright (C) 2003-2006 Robey Pointer <robey@lag.net>
 #
 # This file is part of paramiko.
 #
@@ -30,19 +30,20 @@ import time
 import weakref
 
 from paramiko import util
+from paramiko.auth_handler import AuthHandler
+from paramiko.channel import Channel
 from paramiko.common import *
 from paramiko.compress import ZlibCompressor, ZlibDecompressor
-from paramiko.ssh_exception import SSHException, BadAuthenticationType
-from paramiko.message import Message
-from paramiko.channel import Channel
-from paramiko.sftp_client import SFTPClient
-from paramiko.packet import Packetizer, NeedRekeyException
-from paramiko.rsakey import RSAKey
 from paramiko.dsskey import DSSKey
-from paramiko.kex_group1 import KexGroup1
 from paramiko.kex_gex import KexGex
+from paramiko.kex_group1 import KexGroup1
+from paramiko.message import Message
+from paramiko.packet import Packetizer, NeedRekeyException
 from paramiko.primes import ModulusPack
-from paramiko.auth_handler import AuthHandler
+from paramiko.rsakey import RSAKey
+from paramiko.server import ServerInterface
+from paramiko.sftp_client import SFTPClient
+from paramiko.ssh_exception import SSHException, BadAuthenticationType, ChannelException
 
 # these come from PyCrypt
 #     http://www.amk.ca/python/writing/pycrypt/
@@ -50,7 +51,7 @@ from paramiko.auth_handler import AuthHandler
 # PyCrypt compiled for Win32 can be downloaded from the HashTar homepage:
 #     http://nitace.bsd.uchicago.edu:8080/hashtar
 from Crypto.Cipher import Blowfish, AES, DES3
-from Crypto.Hash import SHA, MD5, HMAC
+from Crypto.Hash import SHA, MD5
 
 
 # for thread cleanup
@@ -73,8 +74,6 @@ class SecurityOptions (object):
     If you try to add an algorithm that paramiko doesn't recognize,
     C{ValueError} will be raised.  If you try to assign something besides a
     tuple to one of the fields, C{TypeError} will be raised.
-
-    @since: ivysaur
     """
     __slots__ = [ 'ciphers', 'digests', 'key_types', 'kex', 'compression', '_transport' ]
 
@@ -110,7 +109,8 @@ class SecurityOptions (object):
         if type(x) is not tuple:
             raise TypeError('expected tuple or list')
         possible = getattr(self._transport, orig).keys()
-        if len(filter(lambda n: n not in possible, x)) > 0:
+        forbidden = filter(lambda n: n not in possible, x)
+        if len(forbidden) > 0:
             raise ValueError('unknown cipher')
         setattr(self._transport, name, x)
 
@@ -149,7 +149,7 @@ class Transport (threading.Thread):
     """
 
     _PROTO_ID = '2.0'
-    _CLIENT_ID = 'paramiko_1.5.2'
+    _CLIENT_ID = 'paramiko_1.6.2'
 
     _preferred_ciphers = ( 'aes128-cbc', 'blowfish-cbc', 'aes256-cbc', '3des-cbc' )
     _preferred_macs = ( 'hmac-sha1', 'hmac-md5', 'hmac-sha1-96', 'hmac-md5-96' )
@@ -245,25 +245,39 @@ class Transport (threading.Thread):
             self.sock.settimeout(0.1)
         except AttributeError:
             pass
+
         # negotiated crypto parameters
         self.packetizer = Packetizer(sock)
         self.local_version = 'SSH-' + self._PROTO_ID + '-' + self._CLIENT_ID
         self.remote_version = ''
         self.local_cipher = self.remote_cipher = ''
         self.local_kex_init = self.remote_kex_init = None
+        self.local_mac = self.remote_mac = None
+        self.local_compression = self.remote_compression = None
         self.session_id = None
-        # /negotiated crypto parameters
-        self.expected_packet = 0
+        self.host_key_type = None
+        self.host_key = None
+        
+        # state used during negotiation
+        self.kex_engine = None
+        self.H = None
+        self.K = None
+
         self.active = False
         self.initial_kex_done = False
         self.in_kex = False
+        self.authenticated = False
+        self._expected_packet = tuple()
         self.lock = threading.Lock()    # synchronization (always higher level than write_lock)
+
+        # tracking open channels
         self.channels = weakref.WeakValueDictionary()   # (id -> Channel)
         self.channel_events = { }       # (id -> Event)
         self.channels_seen = { }        # (id -> True)
         self.channel_counter = 1
         self.window_size = 65536
         self.max_packet_size = 34816
+
         self.saved_exception = None
         self.clear_to_send = threading.Event()
         self.clear_to_send_lock = threading.Lock()
@@ -271,9 +285,10 @@ class Transport (threading.Thread):
         self.logger = util.get_logger(self.log_name)
         self.packetizer.set_log(self.logger)
         self.auth_handler = None
-        self.authenticated = False
-        # user-defined event callbacks:
-        self.completion_event = None
+        self.global_response = None     # response Message from an arbitrary global request
+        self.completion_event = None    # user-defined event callbacks
+        self.banner_timeout = 15        # how long (seconds) to wait for the SSH banner
+
         # server mode:
         self.server_mode = False
         self.server_object = None
@@ -282,9 +297,6 @@ class Transport (threading.Thread):
         self.server_accept_cv = threading.Condition(self.lock)
         self.subsystem_table = { }
 
-    def __del__(self):
-        self.close()
-
     def __repr__(self):
         """
         Returns a string representation of this object, for debugging.
@@ -309,6 +321,19 @@ class Transport (threading.Thread):
                 out += ' (connecting)'
         out += '>'
         return out
+    
+    def atfork(self):
+        """
+        Terminate this Transport without closing the session.  On posix
+        systems, if a Transport is open during process forking, both parent
+        and child will share the underlying socket, but only one process can
+        use the connection (without corrupting the session).  Use this method
+        to clean up a Transport object without disrupting the other process.
+        
+        @since: 1.5.3
+        """
+        self.sock.close()
+        self.close()
 
     def get_security_options(self):
         """
@@ -319,8 +344,6 @@ class Transport (threading.Thread):
         @return: an object that can be used to change the preferred algorithms
             for encryption, digest (hash), public key, and key exchange.
         @rtype: L{SecurityOptions}
-
-        @since: ivysaur
         """
         return SecurityOptions(self)
 
@@ -496,8 +519,6 @@ class Transport (threading.Thread):
         @return: True if a moduli file was successfully loaded; False
             otherwise.
         @rtype: bool
-
-        @since: doduo
         
         @note: This has no effect when used in client mode.
         """
@@ -522,13 +543,9 @@ class Transport (threading.Thread):
         Close this session, and any open channels that are tied to it.
         """
         self.active = False
-        # since this may be called from __del__, can't assume any attributes exist
-        try:
-            self.packetizer.close()
-            for chan in self.channels.values():
-                chan._unlink()
-        except AttributeError:
-            pass
+        self.packetizer.close()
+        for chan in self.channels.values():
+            chan._unlink()
 
     def get_remote_server_key(self):
         """
@@ -541,7 +558,7 @@ class Transport (threading.Thread):
 
         @raise SSHException: if no session is currently active.
         
-        @return: public key of the remote server.
+        @return: public key of the remote server
         @rtype: L{PKey <pkey.PKey>}
         """
         if (not self.active) or (not self.initial_kex_done):
@@ -553,7 +570,7 @@ class Transport (threading.Thread):
         Return true if this session is active (open).
 
         @return: True if the session is still active (open); False if the
-            session is closed.
+            session is closed
         @rtype: bool
         """
         return self.active
@@ -563,9 +580,11 @@ class Transport (threading.Thread):
         Request a new channel to the server, of type C{"session"}.  This
         is just an alias for C{open_channel('session')}.
 
-        @return: a new L{Channel} on success, or C{None} if the request is
-            rejected or the session ends prematurely.
+        @return: a new L{Channel}
         @rtype: L{Channel}
+        
+        @raise SSHException: if the request is rejected or the session ends
+            prematurely
         """
         return self.open_channel('session')
 
@@ -577,18 +596,20 @@ class Transport (threading.Thread):
         L{connect} or L{start_client}) and authenticating.
 
         @param kind: the kind of channel requested (usually C{"session"},
-            C{"forwarded-tcpip"} or C{"direct-tcpip"}).
+            C{"forwarded-tcpip"} or C{"direct-tcpip"})
         @type kind: str
         @param dest_addr: the destination address of this port forwarding,
             if C{kind} is C{"forwarded-tcpip"} or C{"direct-tcpip"} (ignored
-            for other channel types).
+            for other channel types)
         @type dest_addr: (str, int)
         @param src_addr: the source address of this port forwarding, if
-            C{kind} is C{"forwarded-tcpip"} or C{"direct-tcpip"}.
+            C{kind} is C{"forwarded-tcpip"} or C{"direct-tcpip"}
         @type src_addr: (str, int)
-        @return: a new L{Channel} on success, or C{None} if the request is
-            rejected or the session ends prematurely.
+        @return: a new L{Channel} on success
         @rtype: L{Channel}
+
+        @raise SSHException: if the request is rejected or the session ends
+            prematurely
         """
         chan = None
         if not self.active:
@@ -620,19 +641,25 @@ class Transport (threading.Thread):
         finally:
             self.lock.release()
         self._send_user_message(m)
-        while 1:
+        while True:
             event.wait(0.1);
             if not self.active:
-                return None
+                e = self.get_exception()
+                if e is None:
+                    e = SSHException('Unable to open channel.')
+                raise e
             if event.isSet():
                 break
+        self.lock.acquire()
         try:
-            self.lock.acquire()
-            if not self.channels.has_key(chanid):
-                chan = None
+            if self.channels.has_key(chanid):
+                return chan
         finally:
             self.lock.release()
-        return chan
+        e = self.get_exception()
+        if e is None:
+            e = SSHException('Unable to open channel.')
+        raise e
 
     def open_sftp_client(self):
         """
@@ -656,8 +683,6 @@ class Transport (threading.Thread):
         @param bytes: the number of random bytes to send in the payload of the
             ignored packet -- defaults to a random number from 10 to 41.
         @type bytes: int
-
-        @since: fearow
         """
         m = Message()
         m.add_byte(chr(MSG_IGNORE))
@@ -674,22 +699,23 @@ class Transport (threading.Thread):
         bytes sent or received, but this method gives you the option of forcing
         new keys whenever you want.  Negotiating new keys causes a pause in
         traffic both ways as the two sides swap keys and do computations.  This
-        method returns when the session has switched to new keys, or the
-        session has died mid-negotiation.
+        method returns when the session has switched to new keys.
 
-        @return: True if the renegotiation was successful, and the link is
-            using new keys; False if the session dropped during renegotiation.
-        @rtype: bool
+        @raise SSHException: if the key renegotiation failed (which causes the
+            session to end)
         """
         self.completion_event = threading.Event()
         self._send_kex_init()
-        while 1:
-            self.completion_event.wait(0.1);
+        while True:
+            self.completion_event.wait(0.1)
             if not self.active:
-                return False
+                e = self.get_exception()
+                if e is not None:
+                    raise e
+                raise SSHException('Negotiation failed.')
             if self.completion_event.isSet():
                 break
-        return True
+        return
 
     def set_keepalive(self, interval):
         """
@@ -701,11 +727,9 @@ class Transport (threading.Thread):
         @param interval: seconds to wait before sending a keepalive packet (or
             0 to disable keepalives).
         @type interval: int
-
-        @since: fearow
         """
         self.packetizer.set_keepalive(interval,
-            lambda x=self: x.global_request('keepalive@lag.net', wait=False))
+            lambda x=weakref.proxy(self): x.global_request('keepalive@lag.net', wait=False))
 
     def global_request(self, kind, data=None, wait=True):
         """
@@ -724,8 +748,6 @@ class Transport (threading.Thread):
             request was successful (or an empty L{Message} if C{wait} was
             C{False}); C{None} if the request was denied.
         @rtype: L{Message}
-
-        @since: fearow
         """
         if wait:
             self.completion_event = threading.Event()
@@ -807,8 +829,6 @@ class Transport (threading.Thread):
         
         @raise SSHException: if the SSH2 negotiation fails, the host key
             supplied by the server is incorrect, or authentication fails.
-
-        @since: doduo
         """
         if hostkey is not None:
             self._preferred_keys = [ hostkey.get_name() ]
@@ -896,8 +916,6 @@ class Transport (threading.Thread):
 
         @return: username that was authenticated, or C{None}.
         @rtype: string
-
-        @since: fearow
         """
         if not self.active or (self.auth_handler is None):
             return None
@@ -974,8 +992,9 @@ class Transport (threading.Thread):
         
         @raise BadAuthenticationType: if password authentication isn't
             allowed by the server for this user (and no event was passed in)
-        @raise SSHException: if the authentication failed (and no event was
-            passed in)
+        @raise AuthenticationException: if the authentication failed (and no
+            event was passed in)
+        @raise SSHException: if there was a network error
         """
         if (not self.active) or (not self.initial_kex_done):
             # we should never try to send the password unless we're on a secure link
@@ -993,7 +1012,7 @@ class Transport (threading.Thread):
             return self.auth_handler.wait_for_response(my_event)
         except BadAuthenticationType, x:
             # if password auth isn't allowed, but keyboard-interactive *is*, try to fudge it
-            if not fallback or not 'keyboard-interactive' in x.allowed_types:
+            if not fallback or ('keyboard-interactive' not in x.allowed_types):
                 raise
             try:
                 def handler(title, instructions, fields):
@@ -1010,6 +1029,7 @@ class Transport (threading.Thread):
             except SSHException, ignored:
                 # attempt failed; just raise the original exception
                 raise x
+        return None
 
     def auth_publickey(self, username, key, event=None):
         """
@@ -1037,13 +1057,14 @@ class Transport (threading.Thread):
             complete (whether it was successful or not)
         @type event: threading.Event
         @return: list of auth types permissible for the next stage of
-            authentication (normally empty).
+            authentication (normally empty)
         @rtype: list
         
         @raise BadAuthenticationType: if public-key authentication isn't
-            allowed by the server for this user (and no event was passed in).
-        @raise SSHException: if the authentication failed (and no event was
-            passed in).
+            allowed by the server for this user (and no event was passed in)
+        @raise AuthenticationException: if the authentication failed (and no
+            event was passed in)
+        @raise SSHException: if there was a network error
         """
         if (not self.active) or (not self.initial_kex_done):
             # we should never try to authenticate unless we're on a secure link
@@ -1100,7 +1121,8 @@ class Transport (threading.Thread):
         
         @raise BadAuthenticationType: if public-key authentication isn't
             allowed by the server for this user
-        @raise SSHException: if the authentication failed
+        @raise AuthenticationException: if the authentication failed
+        @raise SSHException: if there was a network error
         
         @since: 1.5
         """
@@ -1119,13 +1141,14 @@ class Transport (threading.Thread):
         (See the C{logging} module for more info.)  SSH Channels will log
         to a sub-channel of the one specified.
 
-        @param name: new channel name for logging.
+        @param name: new channel name for logging
         @type name: str
 
         @since: 1.1
         """
         self.log_name = name
         self.logger = util.get_logger(name)
+        self.packetizer.set_log(self.logger)
 
     def get_log_channel(self):
         """
@@ -1179,6 +1202,21 @@ class Transport (threading.Thread):
             self._preferred_compression = ( 'zlib@openssh.com', 'zlib', 'none' )
         else:
             self._preferred_compression = ( 'none', )
+    
+    def getpeername(self):
+        """
+        Return the address of the remote side of this Transport, if possible.
+        This is effectively a wrapper around C{'getpeername'} on the underlying
+        socket.  If the socket-like object has no C{'getpeername'} method,
+        then C{("unknown", 0)} is returned.
+        
+        @return: the address if the remote host, if known
+        @rtype: tuple(str, int)
+        """
+        gp = getattr(self.sock, 'getpeername', None)
+        if gp is None:
+            return ('unknown', 0)
+        return gp()
 
     def stop_thread(self):
         self.active = False
@@ -1237,9 +1275,9 @@ class Transport (threading.Thread):
         if self.session_id == None:
             self.session_id = h
 
-    def _expect_packet(self, type):
+    def _expect_packet(self, *ptypes):
         "used by a kex object to register the next packet type it expects to see"
-        self.expected_packet = type
+        self._expected_packet = tuple(ptypes)
 
     def _verify_key(self, host_key, sig):
         key = self._key_info[self.host_key_type](Message(host_key))
@@ -1262,9 +1300,9 @@ class Transport (threading.Thread):
             m.add_mpint(self.K)
             m.add_bytes(self.H)
             m.add_bytes(sofar)
-            hash = SHA.new(str(m)).digest()
-            out += hash
-            sofar += hash
+            digest = SHA.new(str(m)).digest()
+            out += digest
+            sofar += digest
         return out[:nbytes]
 
     def _get_cipher(self, name, key, iv):
@@ -1288,7 +1326,7 @@ class Transport (threading.Thread):
             self.packetizer.write_all(self.local_version + '\r\n')
             self._check_banner()
             self._send_kex_init()
-            self.expected_packet = MSG_KEXINIT
+            self._expect_packet(MSG_KEXINIT)
 
             while self.active:
                 if self.packetizer.need_rekey() and not self.in_kex:
@@ -1307,10 +1345,10 @@ class Transport (threading.Thread):
                 elif ptype == MSG_DEBUG:
                     self._parse_debug(m)
                     continue
-                if self.expected_packet != 0:
-                    if ptype != self.expected_packet:
-                        raise SSHException('Expecting packet %d, got %d' % (self.expected_packet, ptype))
-                    self.expected_packet = 0
+                if len(self._expected_packet) > 0:
+                    if ptype not in self._expected_packet:
+                        raise SSHException('Expecting packet from %r, got %d' % (self._expected_packet, ptype))
+                    self._expected_packet = tuple()
                     if (ptype >= 30) and (ptype <= 39):
                         self.kex_engine.parse_next(ptype, m)
                         continue
@@ -1388,30 +1426,31 @@ class Transport (threading.Thread):
     def _check_banner(self):
         # this is slow, but we only have to do it once
         for i in range(5):
-            # give them 5 seconds for the first line, then just 2 seconds each additional line
+            # give them 15 seconds for the first line, then just 2 seconds
+            # each additional line.  (some sites have very high latency.)
             if i == 0:
-                timeout = 5
+                timeout = self.banner_timeout
             else:
                 timeout = 2
             try:
-                buffer = self.packetizer.readline(timeout)
+                buf = self.packetizer.readline(timeout)
             except Exception, x:
                 raise SSHException('Error reading SSH protocol banner' + str(x))
-            if buffer[:4] == 'SSH-':
+            if buf[:4] == 'SSH-':
                 break
-            self._log(DEBUG, 'Banner: ' + buffer)
-        if buffer[:4] != 'SSH-':
-            raise SSHException('Indecipherable protocol version "' + buffer + '"')
+            self._log(DEBUG, 'Banner: ' + buf)
+        if buf[:4] != 'SSH-':
+            raise SSHException('Indecipherable protocol version "' + buf + '"')
         # save this server version string for later
-        self.remote_version = buffer
+        self.remote_version = buf
         # pull off any attached comment
         comment = ''
-        i = string.find(buffer, ' ')
+        i = string.find(buf, ' ')
         if i >= 0:
-            comment = buffer[i+1:]
-            buffer = buffer[:i]
+            comment = buf[i+1:]
+            buf = buf[:i]
         # parse out version string and make sure it matches
-        segs = buffer.split('-', 2)
+        segs = buf.split('-', 2)
         if len(segs) < 3:
             raise SSHException('Invalid SSH banner')
         version = segs[1]
@@ -1612,7 +1651,7 @@ class Transport (threading.Thread):
         if not self.packetizer.need_rekey():
             self.in_kex = False
         # we always expect to receive NEWKEYS now
-        self.expected_packet = MSG_NEWKEYS
+        self._expect_packet(MSG_NEWKEYS)
 
     def _auth_trigger(self):
         self.authenticated = True
@@ -1661,7 +1700,11 @@ class Transport (threading.Thread):
         kind = m.get_string()
         self._log(DEBUG, 'Received global request "%s"' % kind)
         want_reply = m.get_boolean()
-        ok = self.server_object.check_global_request(kind, m)
+        if not self.server_mode:
+            self._log(DEBUG, 'Rejecting "%s" channel request from server.' % kind)
+            ok = False
+        else:
+            ok = self.server_object.check_global_request(kind, m)
         extra = ()
         if type(ok) is tuple:
             extra = ok
@@ -1712,14 +1755,12 @@ class Transport (threading.Thread):
         reason = m.get_int()
         reason_str = m.get_string()
         lang = m.get_string()
-        if CONNECTION_FAILED_CODE.has_key(reason):
-            reason_text = CONNECTION_FAILED_CODE[reason]
-        else:
-            reason_text = '(unknown code)'
+        reason_text = CONNECTION_FAILED_CODE.get(reason, '(unknown code)')
         self._log(INFO, 'Secsh channel %d open FAILED: %s: %s' % (chanid, reason_str, reason_text))
+        self.lock.acquire()
         try:
-            self.lock.aquire()
-            if self.channels.has_key(chanid):
+            self.saved_exception = ChannelException(reason, reason_text)
+            if self.channel_events.has_key(chanid):
                 del self.channels[chanid]
                 if self.channel_events.has_key(chanid):
                     self.channel_events[chanid].set()
